@@ -2,7 +2,7 @@ package Finance::Budget;
 
 use strict;
 use warnings;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 {   use Carp;
     use Text::CSV;
     use Date::Manip;
@@ -84,7 +84,7 @@ sub new
         if @{ $self{recent_history} } < 1 + @{ $self{transaction_types} };
 
     croak 'there must be 1+ categorizer for each transaction type'
-        if @{ $self{transaction_types} } < keys %{ $self{categorizer} };
+        if @{ $self{transaction_types} } > keys %{ $self{categorizer} };
 
     $self{opening_balance} *= 100; # dollars to cents
 
@@ -95,6 +95,26 @@ sub new
     return bless \%self, $class;
 }
 
+sub get_last_occurences {
+    my ($self) = @_;
+
+    my %last_occurred;
+
+    TYPE:
+    for my $type_hr (@{ $self->{transaction_types} })
+    {
+        my $category   = $type_hr->{category};
+        my $base_date  = $type_hr->{base_date};
+
+        next TYPE
+            if not $category;
+
+        $last_occurred{$category} = $base_date->printf('%m/%d/%Y');
+    }
+
+    return \%last_occurred;
+}
+
 sub opening_balance
 {
     my ($self) = @_;
@@ -200,6 +220,9 @@ sub _set_base_dates
             my $category = $categorize->( \%event )
                 || next EVENT;
 
+            next EVENT
+                if ref $base_date_for{$category};
+
             my $date = Date::Manip::Date->new();
 
             my $err = $date->parse($event{date});
@@ -212,17 +235,20 @@ sub _set_base_dates
         }
     }
 
+    CAT:
     for my $category (keys %base_date_for)
     {
         croak sprintf 'failed to find base date for %s in recent history',
             $category
-            if not $base_date_for{$category};
+            if not ref $base_date_for{$category};
 
         for my $type_hr (@{ $self->{transaction_types} })
         {
             if ( $type_hr->{category} eq $category )
             {
                 $type_hr->{base_date} = $base_date_for{$category};
+
+                next CAT;
             }
         }
     }
@@ -311,7 +337,7 @@ sub _build_transactions {
             $amount = $find_amount_cr->(
                 $type_hr->{category},
                 $epoch,
-                $amount,
+                $type_hr->{amount},
             );
 
             $events_for{$epoch} //= [];
@@ -389,6 +415,7 @@ sub _format_currency {
     if ( $cents < 99 )
     {
         $dollars = substr 0, 1, sprintf '0.%02d', $cents;
+        $dollars ||= '0.00';
     }
     else
     {
@@ -404,6 +431,10 @@ use strict;
 use warnings;
 use overload (
     q{""} => \&_stringify,
+    q{<}  => \&_lt,
+    q{==} => \&_eq,
+    q{!=} => sub { !_eq( @_ ) },
+    q{>}  => sub { !_eq( @_ ) && !_lt( @_ ) },
 );
 
 sub new {
@@ -464,6 +495,17 @@ sub _stringify {
     return $self->{event}->{string};
 }
 
+sub _lt {
+    my ($self, $arg) = @_;
+    return $self->{event}->{cents} < $arg;
+}
+
+sub _eq {
+    my ($self, $arg) = @_;
+    return $self->{event}->{cents} == $arg;
+}
+
+
 package Finance::Budget::Chokepoints;
 
 use strict;
@@ -519,6 +561,10 @@ use strict;
 use warnings;
 use overload (
     q{""} => \&_stringify,
+    q{<}  => \&_lt,
+    q{==} => \&_eq,
+    q{!=} => sub { !_eq( @_ ) },
+    q{>}  => sub { !_eq( @_ ) && !_lt( @_ ) },
 );
 
 sub new
@@ -535,6 +581,16 @@ sub _stringify
     return $self->{markup}->( $self->{point} );
 }
 
+sub _lt {
+    my ($self, $arg) = @_;
+    return $self->{point}->{cents} < $arg;
+}
+
+sub _eq {
+    my ($self, $arg) = @_;
+    return $self->{point}->{cents} == $arg;
+}
+
 
 1;
 
@@ -617,23 +673,27 @@ budget.
       }
   );
 
-  printf " Opening Balance: % 28s\n",
+  printf "Opening Balance: % 25s\n",
       $budget->opening_balance();
 
   while ( my $transaction = $budget->next() )
   {
-      print "$transaction\n";
+      print "$transaction\n"
+        if $transaction != 0;
   }
 
   my $chokepoints = $budget->get_chokepoints();
 
-  printf "\n Eye of the needle:\n%s\n\n",
+  printf "\nEye of the needle:\n%s\n\n",
       $chokepoints->eye();
 
-  printf " Chokepoints:\n";
+  printf "Chokepoints:\n";
 
   while ( my $point = $chokepoints->next() )
   {
+      die "negative chokepoint: $point\n"
+          if $point < 0;
+
       print "$point\n";
   }
 
@@ -769,6 +829,13 @@ Get the next transaction.
 
 Returns a chokepoint iterator.
 
+=head2 get_last_occurences()
+
+Returns a hash ref mapping of categories and the last time each one
+occurred as parsed from the recent history.
+
+This might be helpful for debugging your categorizers.
+
 
 =head1 TRANSACTIONS
 
@@ -791,12 +858,25 @@ The transaction object also offers some getter methods.
 
 =back
 
+The transaction objects can be interpolated into strings and they can be used
+in numeric comparisons.
+
+  print "$transaction\n"
+      if $transaction != 0;
+
+
 =head1 CHOKEPOINTS
 
 The chokepoints are the balance minima between the significant expenses and
 the significant income events. You might be interested in iterating through
 these to see where you're going to run into trouble.
 
+The chokepoint objects can be interpolated into strings and they can be used
+in numeric comparisons.
+
+  die "negative chokepoint: $point\n"
+      if $point < 0;
+
 =head2 eye()
 
 This is the "eye" as in threading the needle -- the smallest of the
